__version__ = '0.1.0a2'
__author__ = 'Alon Krymgand Osovsky'
__author_email__ = 'downtown2u@gmail.com'
__description__ = ' '.join((
    'Your personal assistant for everything',
    'competitive-programming-related.',
))
