// {% if user is defined %}{{ user }} - {% endif %}{{ now.ctime() }}
// {{ problem.name }} ({{ problem.url }})
// Generated by cptk

{% if system != 'Windows' -%}
#include <bits/stdc++.h>
{% else -%}
#include <algorithm>
#include <iostream>
#include <string>
#include <vector>
{% endif -%}
using namespace std;

int main() {
    cin.tie(NULL)->sync_with_stdio(false);
}
